@@ -1,9 +1,12 @@
 #include <cmath>
 #include <cstdlib>
+#include <cstdio>
 #include <string>
 #include <algorithm>
 #include <random>
-#include <boost/lexical_cast.hpp>
+
+#define BOOST_DISABLE_ASSERTS
+
 #include <boost/random/exponential_distribution.hpp>
 
 extern "C" {
@@ -18,6 +21,8 @@ extern "C" {
 
 using namespace std;
 
+namespace dfcomb { namespace logistic {
+
 struct datastru{
   double d0, a0, b0, c0;
   vector<double> dose_scale_1, dose_scale_2;
@@ -37,14 +42,6 @@ struct datastru{
   { }
 };
 
-double proba_tox(double x_p, double y_q, double d0, double a0, double b0, double c0);
-void genpopn(datastru * datapt, const vector<vector<double> >& pi);
-void startup(datastru *datapt, const vector<vector<double> >& pi);
-double density(double x,  void *data);
-void estimation(datastru * datapt);
-void take_if_better(datastru * datapt, int& recomdose1, int& recomdose2, int candidate_dose1, int candidate_dose2);
-bool newdfrule(datastru * datapt);
-
 enum para_ty {
   PARA_d0,
   PARA_a0,
@@ -59,345 +56,7 @@ double ESCP, DESP, ARRET, NMIN;
 
 std::minstd_rand r;
 
-R_NativePrimitiveArgType logistic_sim_args[] =
-  {LGLSXP, INTSXP, INTSXP,
-   REALSXP, REALSXP,
-   REALSXP,
-   REALSXP, REALSXP, REALSXP,
-   REALSXP, REALSXP,
-   INTSXP, INTSXP, INTSXP,
-   REALSXP, REALSXP, REALSXP,
-   INTSXP, INTSXP,
-
-   REALSXP, REALSXP, REALSXP,
-   REALSXP};
-const int logistic_sim_nargs = 23;
-
-void logistic_sim(int* tite, int* ndose1, int* ndose2,
-                  double* timefull, double* week_incl,
-                  double* piv,
-                  double* target, double* target_max, double* target_min,
-                  double* prior_1, double* prior_2,
-                  int* ncohort, int* cohort, int* ntrial,
-                  double* escp, double* desp, double* arret,
-                  int* nmin, int* seed,
-
-                  double* nrecdoserat, double* nptsdoserat, double* ntoxrat,
-                  double* inconcrat)
-{
-  try {
-
-  ESCP = *escp;
-  DESP = *desp;
-  ARRET = *arret;
-  NMIN = *nmin;
-  r.seed(*seed);
-
-  TITE = *tite;
-  NDOSE1 = *ndose1;
-  NDOSE2 = *ndose2;
-
-  struct datastru data(NDOSE1, NDOSE2);
-
-  vector<vector<double> > piV(NDOSE1, vector<double>(NDOSE2));
-
-  for(int i=0; i<NDOSE1; i++){
-    for(int j=0; j<NDOSE2; j++){
-      piV[i][j] = piv[i + j*NDOSE1];
-    }
-  }
-
-  if(TITE) {
-    TIMEFULL = *timefull;
-    WEEK_incl = *week_incl;
-  }
-
-  TARGET = *target;
-  TARGET_MIN = *target_min;
-  TARGET_MAX = *target_max;
-  
-  for(int i=0; i<NDOSE1; i++){
-    data.dose_scale_1[i] = log(prior_1[i]/(1-prior_1[i]));
-  }
-
-  for(int i=0; i<NDOSE2; i++){
-    data.dose_scale_2[i] = log(prior_2[i]/(1-prior_2[i]));
-  }
-
-  NCOHORT = *ncohort;
-  COHORT = *cohort;
-  
-  int inconc=0;
-  int ntreated=0;
-  vector<vector<int> > nptsdose(NDOSE1, vector<int>(NDOSE2, 0)),
-    ntox(NDOSE1, vector<int>(NDOSE2, 0)), nrecdose(NDOSE1, vector<int>(NDOSE2, 0));
-  
-  // Trials simulations
-  Progress prog(*ntrial);
-  for(int trial=0; trial<*ntrial; trial++) {
-    data.pat_incl = 0;
-    for(int i=0; i<NDOSE1; i++){
-      for(int j=0; j<NDOSE2; j++){
-        data.y[i][j]=0;
-        data.n[i][j]=0;
-      }
-    }
-
-    data.cdose1 = 0;
-    data.cdose2 = 0;
-
-    // Start-up phase
-    startup(&data, piV);
-
-    // Cohort inclusions and estimations, based on the model
-    while(data.pat_incl < COHORT * NCOHORT){
-      if(prog.check_abort()) {
-        *ntrial = trial;
-        goto aborted;
-      }
-
-      estimation(&data);
-
-      if(newdfrule(&data)) {
-        inconc++;
-        goto trial_end;
-      }
-
-      genpopn(&data, piV);
-    }
-
-    if(TITE) {
-      for(int i=0; i<data.pat_incl; i++){
-        data.time_follow[i] = INFINITY;
-      }
-
-      for(int i=0; i<NDOSE1; i++){
-        for(int j=0; j<NDOSE2; j++){
-          data.y[i][j] = 0;
-        }
-      }
-
-      for(int i=0; i<data.pat_incl; i++){
-        data.delta[i] = data.time_ev[i] <= TIMEFULL;
-        data.time_min[i] = min(data.time_ev[i], TIMEFULL);
-        data.y[data.dose_adm1[i]][data.dose_adm2[i]] += (int)data.delta[i];
-      }
-    }
-
-    estimation(&data);
-    
-    if(newdfrule(&data)) {
-      inconc++;
-      goto trial_end;
-    }
-
-    { int recom1 = -1, recom2 = -1;
-      double closest=0.0;
-      for(int i=0; i<NDOSE1; i++){
-        for(int j=0; j<NDOSE2; j++){
-          if(data.n[i][j]!=0){
-            double dis = data.ptox_targ[i][j];
-            if(dis >= closest){
-              closest = dis;
-              recom1=i;
-              recom2=j;
-            }
-          }
-        }
-      }
-      if(recom1 == -1 || recom2 == -1)
-        throw std::logic_error("Internal error: no recommended dose.");
-      nrecdose[recom1][recom2]++;
-    }
-
-    trial_end:
-
-    ntreated += data.pat_incl;    
-    for(int i=0; i<NDOSE1; i++){
-      for(int j=0; j<NDOSE2; j++){
-        ntox[i][j] += data.y[i][j];
-        nptsdose[i][j] += data.n[i][j];
-      }
-    }
-
-    prog.increment();
-  }
-
-  aborted:
-
-  for(int i=0; i<NDOSE1; i++){
-    for(int j=0; j<NDOSE2; j++){
-      nrecdoserat[i + j*NDOSE1] = (double)nrecdose[i][j] / *ntrial;
-      nptsdoserat[i + j*NDOSE1] = (double)nptsdose[i][j] / *ntrial;
-      ntoxrat[i + j*NDOSE1] = (double)ntox[i][j] / *ntrial;
-    }
-  }
-
-  *inconcrat = (double)inconc / *ntrial;
-
-  }
-  catch (std::logic_error &e) {
-    error("Internal error in dfcomb (details: %s)", e.what());
-  }
-  catch (...) {
-    error("Internal error in dfcomb");
-  }
-
-  return;
-}
-
-R_NativePrimitiveArgType logistic_next_args[] =
-  {LGLSXP, INTSXP, INTSXP,
-   REALSXP,
-   REALSXP, REALSXP, REALSXP,
-   REALSXP, REALSXP,
-   LGLSXP,
-   REALSXP, REALSXP, REALSXP,
-   INTSXP,
-
-   INTSXP,
-   INTSXP, INTSXP, LGLSXP,
-   INTSXP, INTSXP,
-   REALSXP, REALSXP,
-   LGLSXP,
-
-   LGLSXP,
-   REALSXP, REALSXP, REALSXP, 
-   REALSXP, REALSXP };
-const int logistic_next_nargs = 29;
-
-void logistic_next(int* tite, int* ndose1, int* ndose2,
-                   double* timefull,
-                   double* target, double* target_max, double* target_min,
-                   double* prior_1, double* prior_2,
-                   int* trial_end,
-                   double* escp, double* desp, double* arret,
-                   int* nmin,
-
-                   int* pat_incl,
-                   int* cdose1, int* cdose2, int* in_startup,
-                   int* dose_adm1, int* dose_adm2,
-                   double* time_ev, double* time_follow /* Only for tite */,
-                   int* delta /* Only for non-tite */,
-
-                   int* inconc,
-                   double* pi, double* ptox, double* ptox_inf_targ,
-                   double* ptox_targ, double* ptox_sup_targ)
-{
-  try {
-
-  ESCP = *escp;
-  DESP = *desp;
-  ARRET = *arret;
-  NMIN = *nmin;
-
-  TITE = *tite;
-  NDOSE1 = *ndose1;
-  NDOSE2 = *ndose2;
-
-  struct datastru data(NDOSE1, NDOSE2);
-
-  if(TITE) {
-    TIMEFULL = *timefull;
-  }
-
-  TARGET = *target;
-  TARGET_MIN = *target_min;
-  TARGET_MAX = *target_max;
-
-  for(int i=0; i<NDOSE1; i++){
-    data.dose_scale_1[i] = log(prior_1[i]/(1-prior_1[i]));
-  }
-
-  for(int i=0; i<NDOSE2; i++){
-    data.dose_scale_2[i] = log(prior_2[i]/(1-prior_2[i]));
-  }
-
-  COHORT = 1;
-  NCOHORT = *trial_end ? *pat_incl : *pat_incl + 1;
-
-  data.pat_incl = *pat_incl;
-  data.cdose1 = *cdose1;
-  data.cdose2 = *cdose2;
-  for(int i=0; i < *pat_incl; i++) {
-    data.dose_adm1.push_back(dose_adm1[i]);
-    data.dose_adm2.push_back(dose_adm2[i]);
-    data.n[dose_adm1[i]][dose_adm2[i]]++;
-    if(TITE) {
-      data.time_ev.push_back(time_ev[i]);
-      if(*trial_end && time_follow[i] < TIMEFULL)
-        error("dfcomb : the final recommendation cannot be computed when "
-              "all the patients have not been fully followed");
-      data.time_follow.push_back(time_follow[i]);
-      data.time_min.push_back(min(time_ev[i], min(time_follow[i], TIMEFULL)));
-      data.delta.push_back(data.time_ev[i] <= min(time_follow[i], TIMEFULL));
-    } else {
-      data.delta.push_back(delta[i]);
-    }
-    data.y[dose_adm1[i]][dose_adm2[i]] += (int)data.delta[i];
-  }
-
-  if(*in_startup) {
-    if(*trial_end)
-      error("dfcomb : the final recommendation cannot be computed when "
-            "the startup is not finished");
-    if(data.cdose1 < NDOSE1 - 1 && data.cdose2 < NDOSE2 - 1 &&
-       data.y[data.cdose1][data.cdose2] == 0) {
-      *cdose1 = min(NDOSE1 - 1, data.cdose1 + 1);
-      *cdose2 = min(NDOSE2 - 1, data.cdose2 + 1);
-      *inconc = 0;
-      return;
-    }
-    *in_startup = 0;
-  }
-
-  estimation(&data);
-  *inconc = newdfrule(&data);
-
-  if(!*inconc && *trial_end) {
-    int recom1 = -1, recom2 = -1;
-    double closest=0.0;
-    for(int i=0; i<NDOSE1; i++){
-      for(int j=0; j<NDOSE2; j++){
-        if(data.n[i][j]!=0){
-          double dis = data.ptox_targ[i][j];
-          if(dis >= closest){
-            closest = dis;
-            recom1=i;
-            recom2=j;
-          }
-        }
-      }
-    }
-    if(recom1 == -1 || recom2 == -1)
-      throw std::logic_error("Internal error: no recommended dose.");
-    data.cdose1 = recom1;
-    data.cdose2 = recom2;
-  }
-
-  *cdose1 = data.cdose1;
-  *cdose2 = data.cdose2;
-
-  for(int i=0; i<NDOSE1; i++){
-    for(int j=0; j<NDOSE2; j++){
-      pi[i + j*NDOSE1] = data.pi[i][j];
-      ptox[i + j*NDOSE1] = data.ptox[i][j];
-      ptox_inf_targ[i + j*NDOSE1] = data.ptox_inf_targ[i][j];
-      ptox_targ[i + j*NDOSE1] = data.ptox_targ[i][j];
-      ptox_sup_targ[i + j*NDOSE1] = data.ptox_sup_targ[i][j];
-    }
-  }
-
-  }
-  catch (std::logic_error &e) {
-    error("Internal error in dfcomb (details: %s)", e.what());
-  }
-  catch (...) {
-    error("Internal error in dfcomb");
-  }
-}
-
- // Cohort inclusion
+// Cohort inclusion
 void genpopn(datastru * datapt, const vector<vector<double> >& piV){
   vector<double> time_incl_cohort(COHORT);
   double pi_ij = piV[datapt->cdose1][datapt->cdose2];
@@ -409,7 +68,7 @@ void genpopn(datastru * datapt, const vector<vector<double> >& piV){
     datapt->dose_adm2.push_back(datapt->cdose2);
     if(TITE) {
       datapt->delta.push_back(false);
-      double time_btw_incl = exp_rng(r, 1.0/WEEK_incl);
+      double time_btw_incl = exp_rng(r, WEEK_incl);
       time_incl_cohort[i] = time_btw_incl;
       double time_tox = exp_rng(r, -log(1-pi_ij)/TIMEFULL);
       datapt->time_ev.push_back(time_tox);
@@ -470,7 +129,7 @@ void startup(datastru *datapt, const vector<vector<double> >& piV){
     if(datapt->y[datapt->cdose1][datapt->cdose2] != 0){
       break;
     }
-    
+
     if(datapt->cdose1<NDOSE1-1){
       datapt->cdose1++;
     }
@@ -594,7 +253,7 @@ void estimation(datastru * datapt){
       datapt->ptox_targ[i][j]=0;
       datapt->ptox_sup_targ[i][j]=0;
     }
-  }    
+  }
 
   for(int iter=0; iter<nburn+niter; iter++){
     int err=0;
@@ -607,9 +266,8 @@ void estimation(datastru * datapt){
       PARA = PARA_d0;
       err = arms_simple(ninit, &d0L, &d0R, density, datapt, dometrop, &d0prev, &d0samp, u_random);
       if(err>0) {
-        throw std::logic_error(
-          string("arms internal error (d0): ") +
-          boost::lexical_cast<std::string>(err));
+        char buf[100]; sprintf(buf, "%d", err);
+        throw std::logic_error(string("arms internal error (d0): ") + string(buf));
       }
       datapt->d0 = d0samp;
       d0prev2 = d0prev;
@@ -618,9 +276,8 @@ void estimation(datastru * datapt){
       PARA = PARA_a0;
       err = arms_simple(ninit, &a0L, &a0R, density, datapt, dometrop, &a0prev, &a0samp, u_random);
       if(err>0) {
-        throw std::logic_error(
-          string("arms internal error (a0): ") +
-          boost::lexical_cast<std::string>(err));
+        char buf[100]; sprintf(buf, "%d", err);
+        throw std::logic_error(string("arms internal error (a0): ") + string(buf));
       }
       datapt->a0 = a0samp;
       a0prev2 = a0prev;
@@ -629,9 +286,8 @@ void estimation(datastru * datapt){
       PARA = PARA_b0;
       err = arms_simple(ninit, &b0L, &b0R, density, datapt, dometrop, &b0prev, &b0samp, u_random);
       if(err>0) {
-        throw std::logic_error(
-          string("arms internal error (b0): ") +
-          boost::lexical_cast<std::string>(err));
+        char buf[100]; sprintf(buf, "%d", err);
+        throw std::logic_error(string("arms internal error (b0): ") + string(buf));
       }
       datapt->b0 = b0samp;
       b0prev2 = b0prev;
@@ -640,9 +296,8 @@ void estimation(datastru * datapt){
       PARA = PARA_c0;
       err = arms_simple(ninit, &c0L, &c0R, density, datapt, dometrop, &c0prev, &c0samp, u_random);
       if(err>0) {
-        throw std::logic_error(
-          string("arms internal error (c0): ") +
-          boost::lexical_cast<std::string>(err));
+        char buf[100]; sprintf(buf, "%d", err);
+        throw std::logic_error(string("arms internal error (c0): ") + string(buf));
       }
       datapt->c0 = c0samp;
       c0prev2 = c0prev;
@@ -764,3 +419,343 @@ bool newdfrule(datastru * datapt){
   datapt->cdose2 = recomdose2;
   return false;
 }
+
+}}
+
+using namespace dfcomb::logistic;
+
+R_NativePrimitiveArgType logistic_sim_args[] =
+  {LGLSXP, INTSXP, INTSXP,
+   REALSXP, REALSXP,
+   REALSXP,
+   REALSXP, REALSXP, REALSXP,
+   REALSXP, REALSXP,
+   INTSXP, INTSXP, INTSXP,
+   REALSXP, REALSXP, REALSXP,
+   INTSXP, INTSXP,
+
+   REALSXP, REALSXP, REALSXP,
+   REALSXP};
+const int logistic_sim_nargs = 23;
+
+void logistic_sim(int* tite, int* ndose1, int* ndose2,
+                  double* timefull, double* week_incl,
+                  double* piv,
+                  double* target, double* target_max, double* target_min,
+                  double* prior_1, double* prior_2,
+                  int* ncohort, int* cohort, int* ntrial,
+                  double* escp, double* desp, double* arret,
+                  int* nmin, int* seed,
+
+                  double* nrecdoserat, double* nptsdoserat, double* ntoxrat,
+                  double* inconcrat)
+{
+  try {
+
+  ESCP = *escp;
+  DESP = *desp;
+  ARRET = *arret;
+  NMIN = *nmin;
+  r.seed(*seed);
+
+  TITE = *tite;
+  NDOSE1 = *ndose1;
+  NDOSE2 = *ndose2;
+
+  struct datastru data(NDOSE1, NDOSE2);
+
+  vector<vector<double> > piV(NDOSE1, vector<double>(NDOSE2));
+
+  for(int i=0; i<NDOSE1; i++){
+    for(int j=0; j<NDOSE2; j++){
+      piV[i][j] = piv[i + j*NDOSE1];
+    }
+  }
+
+  if(TITE) {
+    TIMEFULL = *timefull;
+    WEEK_incl = *week_incl;
+  }
+
+  TARGET = *target;
+  TARGET_MIN = *target_min;
+  TARGET_MAX = *target_max;
+
+  for(int i=0; i<NDOSE1; i++){
+    data.dose_scale_1[i] = log(prior_1[i]/(1-prior_1[i]));
+  }
+
+  for(int i=0; i<NDOSE2; i++){
+    data.dose_scale_2[i] = log(prior_2[i]/(1-prior_2[i]));
+  }
+
+  NCOHORT = *ncohort;
+  COHORT = *cohort;
+
+  int inconc=0;
+  int ntreated=0;
+  vector<vector<int> > nptsdose(NDOSE1, vector<int>(NDOSE2, 0)),
+    ntox(NDOSE1, vector<int>(NDOSE2, 0)), nrecdose(NDOSE1, vector<int>(NDOSE2, 0));
+
+  // Trials simulations
+  Progress prog(*ntrial);
+  for(int trial=0; trial<*ntrial; trial++) {
+    data.pat_incl = 0;
+    for(int i=0; i<NDOSE1; i++){
+      for(int j=0; j<NDOSE2; j++){
+        data.y[i][j]=0;
+        data.n[i][j]=0;
+      }
+    }
+
+    data.cdose1 = 0;
+    data.cdose2 = 0;
+
+    // Start-up phase
+    startup(&data, piV);
+
+    // Cohort inclusions and estimations, based on the model
+    while(data.pat_incl < COHORT * NCOHORT){
+      if(prog.check_abort()) {
+        *ntrial = trial;
+        goto aborted;
+      }
+
+      estimation(&data);
+
+      if(newdfrule(&data)) {
+        inconc++;
+        goto trial_end;
+      }
+
+      genpopn(&data, piV);
+    }
+
+    if(TITE) {
+      for(int i=0; i<data.pat_incl; i++){
+        data.time_follow[i] = INFINITY;
+      }
+
+      for(int i=0; i<NDOSE1; i++){
+        for(int j=0; j<NDOSE2; j++){
+          data.y[i][j] = 0;
+        }
+      }
+
+      for(int i=0; i<data.pat_incl; i++){
+        data.delta[i] = data.time_ev[i] <= TIMEFULL;
+        data.time_min[i] = min(data.time_ev[i], TIMEFULL);
+        data.y[data.dose_adm1[i]][data.dose_adm2[i]] += (int)data.delta[i];
+      }
+    }
+
+    estimation(&data);
+
+    if(newdfrule(&data)) {
+      inconc++;
+      goto trial_end;
+    }
+
+    { int recom1 = -1, recom2 = -1;
+      double closest=0.0;
+      for(int i=0; i<NDOSE1; i++){
+        for(int j=0; j<NDOSE2; j++){
+          if(data.n[i][j]!=0){
+            double dis = data.ptox_targ[i][j];
+            if(dis >= closest){
+              closest = dis;
+              recom1=i;
+              recom2=j;
+            }
+          }
+        }
+      }
+      if(recom1 == -1 || recom2 == -1)
+        throw std::logic_error("Internal error: no recommended dose.");
+      nrecdose[recom1][recom2]++;
+    }
+
+    trial_end:
+
+    ntreated += data.pat_incl;
+    for(int i=0; i<NDOSE1; i++){
+      for(int j=0; j<NDOSE2; j++){
+        ntox[i][j] += data.y[i][j];
+        nptsdose[i][j] += data.n[i][j];
+      }
+    }
+
+    prog.increment();
+  }
+
+  aborted:
+
+  for(int i=0; i<NDOSE1; i++){
+    for(int j=0; j<NDOSE2; j++){
+      nrecdoserat[i + j*NDOSE1] = (double)nrecdose[i][j] / *ntrial;
+      nptsdoserat[i + j*NDOSE1] = (double)nptsdose[i][j] / *ntrial;
+      ntoxrat[i + j*NDOSE1] = (double)ntox[i][j] / *ntrial;
+    }
+  }
+
+  *inconcrat = (double)inconc / *ntrial;
+
+  }
+  catch (std::logic_error &e) {
+    error("Internal error in dfcomb (details: %s)", e.what());
+  }
+  catch (...) {
+    error("Internal error in dfcomb");
+  }
+
+  return;
+}
+
+R_NativePrimitiveArgType logistic_next_args[] =
+  {LGLSXP, INTSXP, INTSXP,
+   REALSXP,
+   REALSXP, REALSXP, REALSXP,
+   REALSXP, REALSXP,
+   LGLSXP,
+   REALSXP, REALSXP, REALSXP,
+   INTSXP,
+
+   INTSXP,
+   INTSXP, INTSXP,
+   INTSXP, INTSXP,
+   REALSXP, REALSXP,
+   LGLSXP,
+
+   LGLSXP, LGLSXP,
+   REALSXP, REALSXP, REALSXP,
+   REALSXP, REALSXP };
+const int logistic_next_nargs = 29;
+
+void logistic_next(int* tite, int* ndose1, int* ndose2,
+                   double* timefull,
+                   double* target, double* target_max, double* target_min,
+                   double* prior_1, double* prior_2,
+                   int* trial_end,
+                   double* escp, double* desp, double* arret,
+                   int* nmin,
+
+                   int* pat_incl,
+                   int* cdose1, int* cdose2,
+                   int* dose_adm1, int* dose_adm2,
+                   double* time_ev, double* time_follow /* Only for tite */,
+                   int* delta /* Only for non-tite */,
+
+                   int* in_startup, int* inconc,
+                   double* pi, double* ptox, double* ptox_inf_targ,
+                   double* ptox_targ, double* ptox_sup_targ)
+{
+  try {
+
+  ESCP = *escp;
+  DESP = *desp;
+  ARRET = *arret;
+  NMIN = *nmin;
+
+  TITE = *tite;
+  NDOSE1 = *ndose1;
+  NDOSE2 = *ndose2;
+
+  struct datastru data(NDOSE1, NDOSE2);
+
+  if(TITE) {
+    TIMEFULL = *timefull;
+  }
+
+  TARGET = *target;
+  TARGET_MIN = *target_min;
+  TARGET_MAX = *target_max;
+
+  for(int i=0; i<NDOSE1; i++){
+    data.dose_scale_1[i] = log(prior_1[i]/(1-prior_1[i]));
+  }
+
+  for(int i=0; i<NDOSE2; i++){
+    data.dose_scale_2[i] = log(prior_2[i]/(1-prior_2[i]));
+  }
+
+  COHORT = 1;
+  NCOHORT = *trial_end ? *pat_incl : *pat_incl + 1;
+
+  data.pat_incl = *pat_incl;
+  data.cdose1 = *cdose1;
+  data.cdose2 = *cdose2;
+  for(int i=0; i < *pat_incl; i++) {
+    data.dose_adm1.push_back(dose_adm1[i]);
+    data.dose_adm2.push_back(dose_adm2[i]);
+    data.n[dose_adm1[i]][dose_adm2[i]]++;
+    if(TITE) {
+      data.time_ev.push_back(time_ev[i]);
+      if(*trial_end && time_follow[i] < TIMEFULL)
+        error("dfcomb : the final recommendation cannot be computed when "
+              "all the patients have not been fully followed");
+      data.time_follow.push_back(time_follow[i]);
+      data.time_min.push_back(min(time_ev[i], min(time_follow[i], TIMEFULL)));
+      data.delta.push_back(data.time_ev[i] <= min(time_follow[i], TIMEFULL));
+    } else {
+      data.delta.push_back(delta[i]);
+    }
+    data.y[dose_adm1[i]][dose_adm2[i]] += (int)data.delta[i];
+  }
+
+  // Startup
+  if(!*trial_end)
+    for(int d = 0; d < NDOSE1 && d < NDOSE2; d++) {
+      if(data.y[d][d] > 0)
+        break;
+      if(data.n[d][d] == 0) {
+        *cdose1 = *cdose2 = d;
+        *inconc = 0;
+        return;
+      }
+    }
+
+  estimation(&data);
+  *inconc = newdfrule(&data);
+
+  if(!*inconc && *trial_end) {
+    int recom1 = -1, recom2 = -1;
+    double closest=0.0;
+    for(int i=0; i<NDOSE1; i++){
+      for(int j=0; j<NDOSE2; j++){
+        if(data.n[i][j]!=0){
+          double dis = data.ptox_targ[i][j];
+          if(dis >= closest){
+            closest = dis;
+            recom1=i;
+            recom2=j;
+          }
+        }
+      }
+    }
+    if(recom1 == -1 || recom2 == -1)
+      throw std::logic_error("Internal error: no recommended dose.");
+    data.cdose1 = recom1;
+    data.cdose2 = recom2;
+  }
+
+  *cdose1 = data.cdose1;
+  *cdose2 = data.cdose2;
+
+  for(int i=0; i<NDOSE1; i++){
+    for(int j=0; j<NDOSE2; j++){
+      pi[i + j*NDOSE1] = data.pi[i][j];
+      ptox[i + j*NDOSE1] = data.ptox[i][j];
+      ptox_inf_targ[i + j*NDOSE1] = data.ptox_inf_targ[i][j];
+      ptox_targ[i + j*NDOSE1] = data.ptox_targ[i][j];
+      ptox_sup_targ[i + j*NDOSE1] = data.ptox_sup_targ[i][j];
+    }
+  }
+
+  }
+  catch (std::logic_error &e) {
+    error("Internal error in dfcomb (details: %s)", e.what());
+  }
+  catch (...) {
+    error("Internal error in dfcomb");
+  }
+}
